/*
 * BrzozowskiAlgebraic.cpp
 *
 *  Created on: 11. 2. 2014
 *      Author: tomas
 */

#include "BrzozowskiAlgebraic.h"

using namespace alib;
using namespace automaton;
using namespace regexp;

namespace conversions
{

BrzozowskiAlgebraic::BrzozowskiAlgebraic( const FSM & fsm ) : AbstractFAtoREConverter( fsm )
{

}

BrzozowskiAlgebraic::~BrzozowskiAlgebraic( void )
{

}

void BrzozowskiAlgebraic::initEquations( void )
{
    for( const auto & q : m_automaton.getStates( ) )
        m_solver.addSymbol( Symbol( q.getName( ) ) );

    for( const auto & q : m_automaton.getStates( ) )
    {
        if( isInSet( q, m_automaton.getFinalStates( ) ) )
            m_solver.addEquation( Symbol( q.getName( ) ), new RegExpEpsilon( ) );
        else
            m_solver.addEquation( Symbol( q.getName( ) ), new RegExpEmpty( ) );
    }


    for( const auto & p : m_automaton.getStates( ) )
    {
        for( const auto & q : m_automaton.getStates( ) )
        {
            Alternation* alt = new Alternation( );
            alt->getElements( ).push_back( new RegExpEmpty( ) );

            for( const auto & symbol : m_automaton.getInputAlphabet( ) )
                if ( isInSet( TransitionFSM( p, symbol, q ), m_automaton.getTransitions( ) ) )
                    alt->getElements( ).push_back( new RegExpSymbol( symbol.getSymbol( ) ) );

            m_solver.addEquation( Symbol( p.getName( ) ), Symbol( q.getName( ) ), alt );
        }
    }
}

RegExp BrzozowskiAlgebraic::convert( void )
{
    initEquations( );

    Symbol initSymbol( ( * m_automaton.getInitialStates( ).begin( ) ).getName( ) );
    return m_solver.solve( initSymbol );
}

} /* namespace conversions */
