/*
 * Glushkov.h
 *
 *  Created on: 11. 1. 2014
 *      Author: tomas
 */

#ifndef GLUSHKOV_H_
#define GLUSHKOV_H_

#include <map>

#include <alphabet/Symbol.h>
#include <automaton/State.h>
#include <regexp/RegExp.h>

#include "AbstractREtoFAConverter.h"
#include "../shared/GlushkovTraversal.h"

#include "RegExpAlphabet.h"

namespace conversions
{

/**
 * Converts regular expression to finite automata using Glushkov's NFA construction algorithm.
 * Sources:
 *  - Melichar 2.107
 *
 */
class Glushkov : public AbstractREtoFAConverter
{
public:
    Glushkov( const regexp::RegExp & re );
    ~Glushkov( void );
    automaton::FSM convert( void );

private:
    struct NumberedSymbol
    {
        const int m_i;
        const alphabet::Symbol m_alphabetSymbol;
        const automaton::State m_state;

        NumberedSymbol( const regexp::RegExpSymbol * symb, int i );
        bool operator<( const NumberedSymbol & x ) const;
    };

    struct Neighbours
    {
        const regexp::RegExpSymbol * m_first, * m_second;

        Neighbours( const regexp::RegExpSymbol * first, const regexp::RegExpSymbol * second );
        bool operator<( const Neighbours & x ) const;
    };

    void initNumberSymbols( void );
    void constructBeginSymbolSet( void );
    void constructEndSymbolSet( void );
    void constructNeighbourSymbolSet( void );

    std::map<const regexp::RegExpElement*, NumberedSymbol> m_numberedSymbols;
    std::set<NumberedSymbol> m_beginSymbolSet, m_endSymbolSet;
    std::set<GlushkovTraversal::Neighbours> m_neighbourSymbolSet;

};

} /* namespace conversions */

#endif /* GLUSHKOV_H_ */
