/*
 * EpsilonRemover.cpp
 *
 *  Created on: 16. 1. 2014
 *      Author: tomas
 */

#include "EpsilonRemover.h"

using namespace alib;
using namespace automaton;

namespace epsilon
{

EpsilonRemover::EpsilonRemover( const FSM & fsm ) : m_origFSM( fsm )
{
    m_closure = new EpsilonClosure( fsm );

    for( const auto & state : fsm.getStates() )
        m_fsm.addState( state );

    for( const auto & state : fsm.getInitialStates() )
        m_fsm.addInitialState( state );

    for( const auto & symbol : fsm.getInputAlphabet() )
        m_fsm.addInputSymbol( symbol );
}

EpsilonRemover::~EpsilonRemover( void )
{
    delete m_closure;
}

const FSM EpsilonRemover::remove( void )
{
    setTransitions( );
    setFinalStates( );

    return m_fsm;
}

/**
 * Step 1 from Melichar 2.41
 */
void EpsilonRemover::setTransitions( void )
{
    for( const auto & q : m_fsm.getStates( ) )
        for( const auto & p : m_closure->getClosure( q ) )
            for( const auto & transition : m_origFSM.getTransitionsFromState( p ) )
                if( transition.getInput() != Symbol( "" ) )
                    m_fsm.addTransition( q, transition.getInput( ), transition.getTo( ) );
}

/**
 * Step 2 from Melichar 2.41
 */
void EpsilonRemover::setFinalStates( void )
{
    set<State> finalStates;

    for( const auto & q : m_fsm.getStates( ) )
    {
        const set<State> & cl = m_closure->getClosure( q ), & F = m_origFSM.getFinalStates( );
        set<State> intersect;

        set_intersection( cl.begin(), cl.end(), F.begin(), F.end(), std::inserter( intersect, intersect.begin() ) );
        if( intersect.size( ) != 0 )
            finalStates.insert( q );
    }

    for( const auto & q : finalStates )
        m_fsm.addFinalState( q );
}

} /* namespace epsilon */
