/*
 * RegExpFactory.h
 *
 *  Created on: Jan 1, 2014
 *      Author: martin
 */

#ifndef REGEXPFACTORY_H_
#define REGEXPFACTORY_H_

#include "sax/Token.h"
#include "regexp/RegExp.h"

namespace regexp {

using namespace std;
using namespace sax;

/**
 * RegExp builder.
 */
class RegExpFactory {
public:
	/**
	 * Parses the XML in file and returns the RegExp.
	 * @param filename path to the file
	 * @return RegExp
	 */
	static RegExp fromFile(const string& filename);

	/**
	 * Parses the XML and returns the RegExp.
	 * @param str string containing the XML
	 * @return RegExp
	 */
	static RegExp fromString(const string& str);
	
	/**
	 * Parses the XML from stdin and returns the RegExp.
	 * @return RegExp
	 */
	static RegExp fromStdin();
	
	/**
	 * Parses the XML from stream and returns the RegExp.
	 * @return RegExp
	 */
	static RegExp fromStream(std::istream& in);
	
protected:
	/**
	 * Parses the RegExp from list of tokens.
	 * @param tokens XML represented as list of tokens
	 * @return parsed RegExp
	 */
	static RegExp parse(list<Token> tokens);
};

} /* namespace regexp */
#endif /* REGEXPFACTORY_H_ */
