/*
 * ContextFreeGrammar.cpp
 *
 *  Created on: Nov 17, 2013
 *      Author: martin
 */

#include "ContextFreeGrammar.h"

namespace grammar {

ContextFreeGrammar::ContextFreeGrammar() :
		Grammar() {
}

ContextFreeGrammar::ContextFreeGrammar(const set<Symbol>& nonTerminalSymbols, const set<Symbol>& terminalSymbols,
		const Symbol& startSymbol) :
		Grammar(nonTerminalSymbols, terminalSymbols, startSymbol) {
}

bool ContextFreeGrammar::isValidRule(const Rule& rule) const {
	return checkLeftSide(rule.getLeftSide()) && checkRightSide(rule.getRightSide());
}

bool ContextFreeGrammar::checkLeftSide(const list<Symbol>& leftSide) const {
	if (leftSide.size() != 1) {
		return false;
	}

	if (nonTerminalSymbols.find(leftSide.front()) == nonTerminalSymbols.end()) {
		return false;
	}

	return true;
}

bool ContextFreeGrammar::checkRightSide(const list<Symbol>& rightSide) const {

	//check that all symbols are terminal or nonterminal
	for (auto const& symbol : rightSide) {
		if (terminalSymbols.find(symbol) == terminalSymbols.end()
				&& nonTerminalSymbols.find(symbol) == nonTerminalSymbols.end()) {
			return false;
		}
	}

	return true;
}

} /* namespace grammar */
