/*
 * LeftLinearGrammar.h
 *
 *  Created on: Nov 23, 2013
 *      Author: Martin Zak
 */

#ifndef LEFTLINEARGRAMMAR_H_
#define LEFTLINEARGRAMMAR_H_

#include "LinearGrammar.h"

namespace grammar {

/**
 * Left linear grammar. Produces regular language.
 */
class LeftLinearGrammar: public LinearGrammar {
private:
	/**
	 * Checks that left side of the Rule contains one symbol which is nonterminal.
	 * @param leftSide left side of the Rule
	 * @return true when left side contains one nonterminal symbol, false otherwise
	 */
	bool checkLeftSide(const list<Symbol>& leftSide) const;

	/**
	 * Checks that right side of the Rule contains no symbols, one terminal symbol
	 * or nonterminal symbol followed by zero or more terminal symbols.
	 * @param rightSide of the Rule
	 * @return true when right side is valid, false otherwise
	 */
	bool checkRightSide(const list<Symbol>& rightSide) const;
protected:
	/**
	 * @copydoc Grammar::isValidRule(const Rule&) const
	 */
	bool isValidRule(const Rule& rule) const;
public:
	LeftLinearGrammar();
	LeftLinearGrammar(const set<Symbol>& nonTerminalSymbols, const set<Symbol>& terminalSymbols,
			const Symbol& startSymbol);
};

} /* namespace grammar */
#endif /* LEFTLINEARGRAMMAR_H_ */
