/*
 * LeftRegularGrammar.h
 *
 *  Created on: Nov 17, 2013
 *      Author: martin
 */

#ifndef LEFTREGULARGRAMMAR_H_
#define LEFTREGULARGRAMMAR_H_

#include "RegularGrammar.h"

namespace grammar {

/**
 * Left regular grammar. Produces regular language.
 */
class LeftRegularGrammar: public RegularGrammar {
private:
	/**
	 * Checks that left side contains one symbol which is nonterminal.
	 * @param leftSide left side of the Rule
	 * @return true when left side contains one symbol which is nonterminal, false otherwise
	 */
	bool checkLeftSide(const list<Symbol>& leftSide) const;

	/**
	 * Checks that right side of the rule contains one terminal symbol
	 * or nonterminal symbol followed by terminal symbol. Epsilon rules are checked
	 * in RegularGrammar class.
	 * @param rightSide right side of the Rule
	 * @return true when right side of the rule is valid, false otherwise
	 */
	bool checkRightSide(const list<Symbol>& rightSide) const;
protected:
	/**
	 * @copydoc Grammar::isValidRule(const Rule&) const
	 */
	bool isValidRule(const Rule& rule) const;
public:
	LeftRegularGrammar();
	LeftRegularGrammar(const set<Symbol>& nonTerminalSymbols, const set<Symbol>& terminalSymbols,
			const Symbol& startSymbol);
};

} /* namespace grammar */
#endif /* LEFTREGULARGRAMMAR_H_ */
