/*
 * RegExpParser.h
 *
 *  Created on: Nov 23, 2013
 *      Author: martin
 */

#ifndef REGEXPPARSER_H_
#define REGEXPPARSER_H_

#include "RegExp.h"
#include "../sax/Token.h"
#include "RegExpSymbol.h"
#include "RegExpEpsilon.h"
#include "RegExpEmpty.h"

#include "Alternation.h"
#include "Concatenation.h"
#include "Iteration.h"

namespace regexp {

using namespace sax;

/**
 * Parser used to get RegExp from XML parsed into list of tokens.
 */
class RegExpParser {
protected:
	static void parseContent(list<Token>& input, list<RegExpElement*>& elements);
	static RegExpElement* parseElement(list<Token>& input);

	static RegExpEpsilon* parseEpsilon(list<Token>& input);
	static RegExpEmpty* parseEmpty(list<Token>& input);
	static RegExpSymbol* parseSymbol(list<Token> &input);
	static Iteration* parseIteration(list<Token> &input);
	static Alternation* parseAlternation(list<Token> &input);
	static Concatenation* parseConcatenation(list<Token> &input);
	static bool isToken(list<Token> &input, Token::TokenType type, string data);
	static void popToken(list<Token> &input, Token::TokenType type, string data);

public:
	/**
	 * Parses the XML and returns regular expression. The input is destroyed in the process.
	 * @param input XML represented as list of tokens
	 * @return RegExp
	 * @throws ParserException when an error occurs
	 */
	static RegExp parse(list<Token>& input);
};

} /* namespace regexp */
#endif /* REGEXPPARSER_H_ */
