/*
 * Minimize.cpp
 *
 *  Created on: Dec 9, 2013
 *      Author: honza
 */

#include "Minimize.h"

#include <map>
#include <set>
#include <sstream>
#include <iostream>

#include "alphabet/Symbol.h"
#include "automaton/FSM/TransitionFSM.h"

automaton::State fromInteger(int number) {
  std::stringstream ss;
  ss << number;
  return automaton::State(ss.str());
}

automaton::FSM Minimize::minimize(automaton::FSM& fsm) {
	std::map<automaton::State, std::map<alphabet::Symbol, automaton::State > > refactor;

	for(std::set<automaton::State>::const_iterator iter = fsm.getStates().begin(); iter != fsm.getStates().end(); iter++) {
		refactor.insert(std::pair<automaton::State, std::map<alphabet::Symbol, automaton::State> >(*iter, std::map<alphabet::Symbol, automaton::State>()));
	}
	
	for(std::set<automaton::TransitionFSM>::const_iterator iter = fsm.getTransitions().begin(); iter != fsm.getTransitions().end(); iter++) {
		refactor[iter->getFrom()].insert(std::pair<alphabet::Symbol, automaton::State>(iter->getInput(), iter->getTo()));
	}

	std::map<automaton::State, automaton::State> toEquvivalentStates1;//original state to equivalent state
	std::map<std::pair<automaton::State, std::set<std::pair<alphabet::Symbol, automaton::State> > >, std::set<automaton::State> > minimizedTransitionFunction1; //mapped to the original state

	std::map<automaton::State, automaton::State> toEquvivalentStates2;
	std::map<std::pair<automaton::State, std::set<std::pair<alphabet::Symbol, automaton::State> > >, std::set<automaton::State> > minimizedTransitionFunction2;
	
	for(auto iter = fsm.getStates().begin(); iter != fsm.getStates().end(); iter++) {
		if(fsm.getFinalStates().count(*iter) == 0) { // not a final state
			toEquvivalentStates2.insert(std::pair<automaton::State, automaton::State>(*iter, automaton::State("0")));
		} else {
			toEquvivalentStates2.insert(std::pair<automaton::State, automaton::State>(*iter, automaton::State("1")));
		}
	}

	for(auto iter = refactor.begin(); iter != refactor.end(); iter++) {
		automaton::State from = toEquvivalentStates2.find(iter->first)->second;
		std::set<std::pair<alphabet::Symbol, automaton::State> > transitionFunction;

		for(auto iter2 = iter->second.begin(); iter2 != iter->second.end(); iter2++) {
			transitionFunction.insert(std::pair<alphabet::Symbol, automaton::State>(iter2->first, toEquvivalentStates2.find(iter2->second)->second));
		}
		
		std::pair<automaton::State, std::set<std::pair<alphabet::Symbol, automaton::State> > > key(from, transitionFunction);
		
		if(minimizedTransitionFunction2.find(key) == minimizedTransitionFunction2.end()) {
			minimizedTransitionFunction2.insert(std::pair<std::pair<automaton::State, std::set<std::pair<alphabet::Symbol, automaton::State> > >, std::set<automaton::State> >(key, std::set<automaton::State>()));
		}
		minimizedTransitionFunction2[key].insert(iter->first);
	}
	
	do {
		toEquvivalentStates1 = toEquvivalentStates2;
		minimizedTransitionFunction1 = minimizedTransitionFunction2;

		toEquvivalentStates2.clear();
		minimizedTransitionFunction2.clear();
		
		int number = 0;
		for(auto iter = minimizedTransitionFunction1.begin(); iter != minimizedTransitionFunction1.end(); iter++) {
			for(auto iter2 = iter->second.begin(); iter2 != iter->second.end(); iter2++) {
				toEquvivalentStates2.insert(std::pair<automaton::State, automaton::State>(*iter2, fromInteger(number)));
			}
			number++;
		}

		for(auto iter = refactor.begin(); iter != refactor.end(); iter++) {
			automaton::State from = toEquvivalentStates2.find(iter->first)->second;
			std::set<std::pair<alphabet::Symbol, automaton::State> > transitionFunction;
			
			for(auto iter2 = iter->second.begin(); iter2 != iter->second.end(); iter2++) {
				transitionFunction.insert(std::pair<alphabet::Symbol, automaton::State>(iter2->first, toEquvivalentStates2.find(iter2->second)->second));
			}
			
			std::pair<automaton::State, std::set<std::pair<alphabet::Symbol, automaton::State> > > key(from, transitionFunction);
			
			if(minimizedTransitionFunction2.find(key) == minimizedTransitionFunction2.end()) {
				minimizedTransitionFunction2.insert(std::pair<std::pair<automaton::State, std::set<std::pair<alphabet::Symbol, automaton::State> > >, std::set<automaton::State> >(key, std::set<automaton::State>()));
			}

			minimizedTransitionFunction2[key].insert(iter->first);
			
		}

	} while(minimizedTransitionFunction1.size() != minimizedTransitionFunction2.size());

	automaton::FSM result;
	for(auto iter = fsm.getInputAlphabet().begin(); iter != fsm.getInputAlphabet().end(); iter++) {
		result.addInputSymbol(*iter);
	}
	
	for(auto iter = minimizedTransitionFunction2.begin(); iter != minimizedTransitionFunction2.end(); iter++) {
		result.addState(iter->first.first);
		if(fsm.getFinalStates().find(*(iter->second.begin())) != fsm.getFinalStates().end()) {//TODO improve using intersection
			result.addFinalState(iter->first.first);
		}
		if(iter->second.find(*(fsm.getInitialStates().begin())) != iter->second.end()) { //TODO improve using intersection
			result.addInitialState(iter->first.first);
		}
	}
	
	for(auto iter = minimizedTransitionFunction2.begin(); iter != minimizedTransitionFunction2.end(); iter++) {
		for(auto iter2 = iter->first.second.begin(); iter2 != iter->first.second.end(); iter2++) {
			result.addTransition(iter->first.first, iter2->first, iter2->second);
		}
	}
	
	return result;
}
